"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeTCache = exports.resolveTCache = exports.missingTCacheTarget = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const util_1 = require("../util/util");
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const cache_parsing_1 = require("./cache-parsing");
const managed_json_1 = require("../file-formats/common/managed-json");
let lastReadCache = null;
let tCacheTargetMiss = false;
function resolveCachePath(args) {
    const cacheDir = args.cacheDir;
    (0, util_1.checkDir)(cacheDir, { errorHint: "cacheDir" });
    const baseName = path_1.default.basename(args.srcFile);
    const cacheName = `attranslate-cache_from-${args.srcLng}_to-${args.targetFormat}_src-${baseName}.json`;
    return path_1.default.resolve(cacheDir, cacheName);
}
function getCacheWarning(args) {
    return `Do not edit this file manually! You may want to edit '${args.srcFile}' instead.`;
}
function missingTCacheTarget() {
    return tCacheTargetMiss;
}
exports.missingTCacheTarget = missingTCacheTarget;
function resolveTCache(src, args) {
    const cachePath = resolveCachePath(args);
    if (!(0, fs_1.existsSync)(cachePath)) {
        return null;
    }
    const tCache = (0, cache_parsing_1.parseTCache)({ cachePath });
    lastReadCache = tCache;
    const internalCache = new Map();
    Object.keys(tCache.entries).forEach((key) => {
        const entry = tCache.entries[key];
        const internalEntry = cacheEntryToInternalEntry(key, entry, src, args);
        internalCache.set(key, internalEntry);
    });
    return internalCache;
}
exports.resolveTCache = resolveTCache;
function writeTCache(results, args) {
    const cachePath = resolveCachePath(args);
    console.info(`Write cache ${(0, util_1.getDebugPath)(cachePath)}`);
    const cache = {
        "generated-by-attranslate": getCacheWarning(args),
        source: args.srcFile,
        version: { major: 1, minor: 0 },
        entries: {},
    };
    results.newSrcCache.forEach((value, key) => {
        cache.entries[key] = internalEntryToCacheEntry(key, value, args);
    });
    (0, managed_json_1.writeManagedJson)({ path: cachePath, object: cache });
}
exports.writeTCache = writeTCache;
function getTargetId(args) {
    return args.targetLng;
}
function cacheEntryToInternalEntry(key, entry, src, args) {
    const targetId = getTargetId(args);
    const target = entry.targets.find((target) => target.id === targetId);
    if (!target) {
        tCacheTargetMiss = true;
        return entry.value;
    }
    const srcEntry = src.get(key);
    if (srcEntry !== entry.value) {
        return entry.value;
    }
    if (target.state !== "needs_update") {
        return entry.value;
    }
    else {
        // This is a case where we want to enforce an update even if the cache value is up-to-date.
        return srcEntry !== null ? null : " ";
    }
}
function internalEntryToCacheEntry(key, internalEntry, args) {
    const oldEntry = lastReadCache === null || lastReadCache === void 0 ? void 0 : lastReadCache.entries[key];
    const valueChanged = oldEntry
        ? oldEntry.value !== internalEntry
        : false;
    const targetId = getTargetId(args);
    const newTarget = {
        id: targetId,
        state: "frozen",
    };
    let oldTargetIncluded = false;
    const newTargets = !!oldEntry
        ? oldEntry.targets.map((oldTarget) => {
            if (oldTarget.id === targetId) {
                oldTargetIncluded = true;
                return newTarget;
            }
            else if (valueChanged) {
                return { ...oldTarget, state: "needs_update" };
            }
            else {
                return oldTarget;
            }
        })
        : [];
    if (!oldTargetIncluded) {
        newTargets.push(newTarget);
    }
    return {
        value: internalEntry,
        targets: newTargets,
    };
}
