"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseTCache = void 0;
const util_1 = require("../util/util");
const managed_json_1 = require("../file-formats/common/managed-json");
function parseTCache(context) {
    const rawCache = (0, managed_json_1.readRawJson)(context.cachePath).object;
    const { version, entries } = rawCache;
    if (!version) {
        logCacheError(`version is falsy`, context);
    }
    if (version.major !== 1) {
        logCacheError(`Version '${version.major}' is not supported`, context);
    }
    if (!entries) {
        logCacheError(`entries is falsy`, context);
    }
    Object.keys(entries).forEach((key) => {
        const entry = entries[key];
        validateTCacheEntry(entry, { ...context, key });
    });
    return { version, entries };
}
exports.parseTCache = parseTCache;
function logCacheError(rawMsg, context) {
    const msg = context.key
        ? `An error occurred in cache-entry '${context.key}': ${rawMsg}`
        : `A cache error occurred: ${rawMsg}`;
    (0, util_1.logFatal)(`${msg}. You may try to delete ${(0, util_1.getDebugPath)(context.cachePath)} and then re-run this tool.`);
}
function validateTCacheEntry(entry, context) {
    const { value, targets } = entry;
    if (value === undefined) {
        logCacheError(`value is undefined`, context);
    }
    if (!targets || !Array.isArray(targets) || !targets.length) {
        logCacheError(`targets are invalid`, context);
    }
    targets.forEach((target) => {
        validateTCacheTarget(target, context);
    });
    return { value, targets };
}
function validateTCacheTarget(target, context) {
    const { id, state } = target;
    if (!id) {
        logCacheError(`id is falsy`, context);
    }
    if (!state) {
        logCacheError(`state is falsy`, context);
    }
    return { id, state };
}
